/*
 * main.c
 *
 *  Created on: Aug 19, 2011
 *      Author: cberg
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "LEDControl.h"

#define stand_int 0x0F; //standard intensity

static const uint8_t inner_delay = 250;

void outer_set(uint8_t val)
{
	sdp8(REG_DIGIT2, val);
	sdp8(REG_DIGIT3, val);
	sdp8(REG_DIGIT4, val);
	sdp8(REG_DIGIT5, val);
	sdp8(REG_DIGIT6, val);
	sdp8(REG_DIGIT7, val);
}
void inner_set(uint8_t val)
{
	sdp8(REG_DIGIT0, val);
	sdp8(REG_DIGIT1, val);
}
void outer_on()
{
	sdp8(REG_DIGIT0, 0xFF);
	sdp8(REG_DIGIT1, 0xFF);
	sdp8(REG_DIGIT2, 0xFF);
	sdp8(REG_DIGIT3, 0xFF);
	sdp8(REG_DIGIT4, 0xFF);
	sdp8(REG_DIGIT5, 0xFF);
	sdp8(REG_DIGIT6, 0xFF);
	sdp8(REG_DIGIT7, 0xFF);
}
void inner_on()
{
	sdp8(REG_DIGIT0, 0xFF);
	sdp8(REG_DIGIT1, 0xFF);
}
void inner_off()
{
	sdp8(REG_DIGIT0, 0x00);
	sdp8(REG_DIGIT1, 0x00);
}

void glow()
{
	//all_set(0xFF); //All On
	outer_set(0x00); //outer off - we only want the inside glowing
	inner_set(0xFF);
	static uint8_t dir = 0x00; //0=down,1=up
	static uint8_t val = stand_int;
	sdp8(REG_INTENSITY, val);
	_delay_ms(100);
	if (dir == 0x00)//down
		val--;
	else//up
		val++;
	if (val == 0x01) //prevents a value of 0 from being used
		dir = 1;
	else if (val == 0x0F)
		dir = 0;
}
void activate_blade()
{
	sdp8(REG_INTENSITY, 0x0f); //full intensity
	all_set(0x00);
	uint8_t disp = 0;
	for (uint8_t segment = 1; segment <= 8; segment++)
	{
		disp=0;
		for (uint8_t i = 0; i < 8; i++)//sequentially light led's one through 8
		{
			disp += 1<<i;
			sdp8(segment, disp);
			if (segment < 3) //longer delay for inner - otherwise it appears to be running faster
				_delay_ms(45);
			else
				_delay_ms(15);
		}
	}
	//small pause before dropping into next mode
	_delay_ms(250);
}
void deactivate_blade()
{
	sdp8(REG_INTENSITY, 0x0f); //full intensity
	all_set(0xFF); //turn all LEDs on
	uint8_t disp = 0;
	for (uint8_t segment = 8; segment >= 3; segment--) //only deactivating the outer ring
	{
		disp=0xFF;
		for (uint8_t i = 8; i> 0; i--)//sequentially light led's one through 8
		{
			disp -= 1<<(i-1);
			sdp8(segment, disp);
			_delay_ms(15);
		}
	}
	//small pause before dropping into next mode
	_delay_ms(250);
}
void startup_seq()
{
	sdp8(REG_INTENSITY, 0x0f); //full intensity
	all_set(0x00); //turn all leds off
	uint8_t disp = 0;
	for (uint8_t i = 0; i < 8; i++)
	{
		disp += 1<<i;
		sdp8(REG_DIGIT0, disp);
		_delay_ms(inner_delay);
	}
	disp = 0;
	for (uint8_t i = 0; i < 8; i++)
	{
		disp += 1<<i;
		sdp8(REG_DIGIT1, disp);
		_delay_ms(inner_delay);
	}
}
void shutdown_seq()
{
	sdp8(REG_INTENSITY, 0x0f); //full intensity
	all_set(0xFF);//turn all leds on
	uint8_t disp = 0xFF;
	for (uint8_t i = 0; i < 8; i++)
	{
		disp -= 1<<i;
		sdp8(REG_DIGIT0, disp);
		_delay_ms(inner_delay);
	}

	disp = 0xFF;
	for (uint8_t i = 0; i < 8; i++)
	{
		disp -= 1<<i;
		sdp8(REG_DIGIT1, disp);
		_delay_ms(inner_delay);
	}
}

void led_test()
{
	uint8_t disp = 0;
	all_set(0x00); //turn all off
	for (uint8_t segment = 1; segment <= 8; segment++)
	{

		disp=0;
		for (uint8_t i = 0; i < 8; i++)//sequentially light led's one through 8
		{
			disp = 1<<i;
			sdp8(segment, disp);
			_delay_ms(50);
		}
	}
}

void pulse()
{
	//all_set(0xFF);
	static uint8_t step = 0;
	switch( step)
	{

		case 1:
			outer_set(0x33);
			break;
		case 3:
			outer_set(0x66);//0xea
			break;
		case 5:
			outer_set(0xcc);//0xba
			break;
		case 7:
			outer_set(0x99);//0xae
			break;
		default:
				outer_set(0xFF);
				break;
	}
	_delay_ms(30);
	step++;
	if (step > 7)
		step = 0;
}
void pulse2()
{
	static uint8_t step = 0;
	switch(step)
	{
		case 1:
			outer_set(0x88);
			break;
		case 3:
			outer_set(0xCC);
			break;
		case 5:
			outer_set(0xEE);
			break;
		case 7:
			outer_set(0xFF);
			break;
		case 9:
			outer_set(0x44);
			break;
		case 11:
			outer_set(0x66);
			break;
		case 13:
			outer_set(0x77);
			break;
		case 15:
			outer_set(0xFF);
			break;
		case 17:
			outer_set(0x22);
			break;
		case 19:
			outer_set(0x33);
			break;
		case 21:
			outer_set(0xBB);
			break;
		case 23:
			outer_set(0xFF);
			break;
		case 25:
			outer_set(0x11);
			break;
		case 27:
			outer_set(0x99);
			break;
		case 29:
			outer_set(0xDD);
			break;
		default:
			outer_set(0xFF);
			break;
	}
	_delay_ms(10);
	step++;
	if (step > 30)
		step = 0;
}

#define BUTTON_PORT	PORTB
#define BUTTON_PIN	PINB
#define BUTTON_BIT	PB3
#define DOCK_BIT PB4
void pin_setup(void)
{
	/* turn on internal pull-up resistor for the switch */
	BUTTON_PORT |= _BV(BUTTON_BIT)| _BV(DOCK_BIT);
}
void int_enable(void)
{
	sei();
	TIMSK = (1<<OCIE0A); //enable Timer/Counter0 Output Compare Match A
	//need to set 256 prescale
	//also set WGM02
	//timer appears to run the same way it does on the ATMega328 - we should be ok leaving this
	TCCR0B = (1<<CS02)| (1<<WGM02);//256 prescale  - see table 15-9 on documentation
	OCR0A = 254; //compare value - interrupt will trigger at 1.25Khz
}
volatile uint8_t DockState = 2;
volatile uint8_t DockChanged = 0;
volatile uint8_t ModeButtonChange = 0;
ISR(TIMER0_COMPA_vect)
{
	//portb.3 = state switch
	//portb.4 = dock switch
	if ((DockChanged == 0) && (ModeButtonChange == 0)) //we only want to poll pins if the change state is 0 - indicates main loop has accepted change
	{
		if (DockState == 1 || DockState == 2) //set = undocked
		{
			if (bit_is_set(BUTTON_PIN,DOCK_BIT))
			{
				DockChanged = 1;
				DockState = 0;
			}
		}
		if (DockState == 0 || DockState == 2) //clear = docked
		{
			if (bit_is_clear(BUTTON_PIN,DOCK_BIT))
			{
				DockChanged = 1;
				DockState = 1;
			}
		}
		if(DockChanged == 0) //only evaluate button if not changing dock state
		{
			if (bit_is_clear(BUTTON_PIN,BUTTON_BIT))
			{
				ModeButtonChange = 1; //signal to main procedure that button has been pressed
			}
		}
	}
}


int main(void)
{
	pin_setup(); //enable buttons - turn on pull-ups immediately
	SPI_MasterInit();
	initsdp8();

	int_enable();
	_delay_ms(500); //wait a bit for interrupts to start
	uint8_t dockstate_local;
	uint8_t dockchange_local;
	uint8_t modebutton_local;
	uint8_t mode = 0;

	while(1)
		{
			dockstate_local = DockState;
			dockchange_local = DockChanged;
			modebutton_local = ModeButtonChange;
			//dockstate_local = (bit_is_set(BUTTON_PIN,DOCK_BIT));
			if (dockchange_local == 1 && dockstate_local == 1) //startup animation
			{
				cli();
				startup_seq();
				//_delay_ms(500);
				DockChanged = 0;
				sei();
			}
			if (dockchange_local == 1 && dockstate_local == 0)
			{
				cli();
				activate_blade();
				DockChanged = 0;
				sei();
			}
			if (modebutton_local == 1)
			{
				cli();
				ModeButtonChange = 0;//clear button push
				_delay_ms(100); //delay to avoid several button hits
				if (dockstate_local == 0)
				{

					if (mode == 0)
					{
						deactivate_blade();
						mode = 1;
					}
					else
					{
						activate_blade();
						mode = 0;
					}
				}
				sei();
			}
			if (dockstate_local == 1) //not docked
			{
				glow();
			}
			else
			{
				if (mode == 0)
					pulse();
				else
					glow();
			}
		}

}
